package com.dayou.brokerage.config;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.dayou.brokerage.validator.BrokerageValidator;
import com.dayou.common.BaseEntity;
import com.dayou.entity.BrokerageBaseAmount;
import com.dayou.entity.BrokerageMarketerRatio;
import com.dayou.enums.GlobalConfigEnum;
import com.dayou.enums.MergeKeyInterface;
import com.dayou.service.IBrokerageBaseAmountService;
import com.dayou.service.IBrokerageMarketerRatioService;
import com.dayou.service.IGlobalConfigService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.support.DefaultListableBeanFactory;
import org.springframework.context.ApplicationContext;

import javax.annotation.Nullable;
import java.math.BigDecimal;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.stream.Collectors;


/**
 * 类说明：市场人员提成系数表
 *
 * @author: wucl
 * @since: 2023/3/6
 * created with IntelliJ IDEA.
 */
public class LandBrokerageDataConfiguration extends BrokerageMarketerRatio implements MergeKeyInterface {


    @Autowired
    private ApplicationContext applicationContext;

    @Autowired
    private IBrokerageMarketerRatioService brokerageMarketerRatioService;

    @Autowired
    private IGlobalConfigService globalConfigService;

    @Autowired
    private IBrokerageBaseAmountService brokerageBaseAmountService;

    /**
     * 由项目类型和项目来源决定的提成系数集合
     */
    public  ConcurrentHashMap<String, BigDecimal> ratioList = new ConcurrentHashMap<String,BigDecimal>(256);

    /**
     * 参与人基础提成金额
     */
    public ConcurrentHashMap<Long,BigDecimal> userBaseBrokerageAmounts = new ConcurrentHashMap<>(256);

    @Nullable
    public BigDecimal marketerBrokeragePercentage ;

    @Nullable
    public BigDecimal monthOfYear;

    @Nullable
    public BigDecimal nonMarketerMonthBrokerage;

    @Nullable
    public BigDecimal participantBrokeragePercentage;

    public void reloadLandBrokerageData(){
        DefaultListableBeanFactory defaultListableBeanFactory = (DefaultListableBeanFactory) applicationContext.getAutowireCapableBeanFactory();
        defaultListableBeanFactory.destroySingleton("landBrokerageDataConfiguration");
        defaultListableBeanFactory.getBean("landBrokerageDataConfiguration", LandBrokerageDataConfiguration.class);
    }

    public void loadRatioList(){
        List<BrokerageMarketerRatio> list = brokerageMarketerRatioService.list(new LambdaQueryWrapper<BrokerageMarketerRatio>()
                .eq(BaseEntity::getDeleted, Boolean.FALSE));
        list.stream().forEach(x->{
            ratioList.put(x.mergeKey(),new BigDecimal(x.getRatio().toString()));
        });

        nonMarketerMonthBrokerage = globalConfigService.getMathValue(GlobalConfigEnum.FINANCE.getCode(),
                GlobalConfigEnum.Finance.NON_MARKETER_MONTH_BROKERAGE.getCode());

        BigDecimal month = globalConfigService.getMathValue(GlobalConfigEnum.SYS.getCode(),
                GlobalConfigEnum.Sys.MONTH_OF_YEAR.getCode());

        monthOfYear = month.intValue()==0?new BigDecimal("10"):month;

        marketerBrokeragePercentage = globalConfigService.getMathValue(GlobalConfigEnum.FINANCE.getCode(),
                GlobalConfigEnum.Finance.MARKETER_BROKERAGE_PERCENTAGE.getCode());

        participantBrokeragePercentage = globalConfigService.getMathValue(GlobalConfigEnum.FINANCE.getCode(),
                GlobalConfigEnum.Finance.PARTICIPANT_BROKERAGE_PERCENTAGE.getCode());

        List<BrokerageBaseAmount> baseAmounts = brokerageBaseAmountService.list();

        baseAmounts.stream().forEach(x->{
            userBaseBrokerageAmounts.put(x.getUserId(), x.getBaseAmount());
        });
    }
}
